"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationResult = void 0;
class ValidationResult {
    constructor(validationMessagesMap) {
        this.validationMessagesMap = validationMessagesMap;
    }
    getValidationMessagesMap() {
        return this.validationMessagesMap;
    }
    getAllMessages() {
        return this.flatten(this.validationMessagesMap);
    }
    flatten(map) {
        return Object.values(map).flat();
    }
    getAllMessagesOfType(type) {
        return this.flatten(this.filterByType(type));
    }
    getAllValidMessages() {
        return this.flatten(this.filterValid());
    }
    getAllInvalidMessages() {
        return this.flatten(this.filterInvalid());
    }
    getAllMessagesBy(fcn) {
        return this.flatten(this.filterBy(fcn));
    }
    filterByType(type) {
        return this.filterBy(msg => msg.type.name === type.name);
    }
    filterValid() {
        return this.filterBy(msg => msg.type.isValid);
    }
    filterInvalid() {
        return this.filterBy(msg => !msg.type.isValid);
    }
    filterBy(fcn) {
        return Object.keys(this.validationMessagesMap).reduce((res, key) => {
            const filteredMsgs = this.validationMessagesMap[key].filter(msg => fcn(msg));
            if (filteredMsgs.length > 0) {
                res[key] = filteredMsgs;
            }
            return res;
        }, {});
    }
}
exports.ValidationResult = ValidationResult;
